@@ -1,5 +1,5 @@
 /* Implementation of utility functions used in test programs.
-   Copyright (C) 2001-2009 Roberto Bagnara <bagnara@cs.unipr.it>
+   Copyright (C) 2001-2010 Roberto Bagnara <bagnara@cs.unipr.it>
 
 This file is part of the Parma Watchdog Library (PWL).
 
@@ -43,9 +43,12 @@ uncaught_exception_handler() {
   exit(1);
 }
 
-#ifdef HAVE_SIGINFO_T
+#if PWL_HAVE_DECL_SIGACTION
+
+#if defined(PWL_HAVE_SIGINFO_T) && defined(SA_SIGINFO)
+
 void
-fpe_handler(int sig, siginfo_t* sip, void*) {
+fpe_sigaction(int sig, siginfo_t* sip, void*) {
   if (sig != SIGFPE) {
     std::cerr << "fpe_handler called on signal different from SIGFPE"
 	      << std::endl;
@@ -109,11 +112,52 @@ fpe_handler(int sig, siginfo_t* sip, void*) {
       std::cerr << "FE_INVALID ";
 #endif
     std::cerr << std::endl;
-#endif
+#endif // defined(PWL_HAVE_FENV_H)
   }
   exit(1);
 }
-#endif // defined(HAVE_SIGINFO_T)
+
+#else // !defined(PWL_HAVE_SIGINFO_T) || !defined(SA_SIGINFO)
+
+void
+fpe_handler(int sig) {
+  if (sig != SIGFPE) {
+    std::cerr << "fpe_handler called on signal different from SIGFPE"
+	      << std::endl;
+    exit(1);
+  }
+  std::cerr << "SIGFPE caught"
+            << std::endl;
+#if defined(PWL_HAVE_FENV_H)
+  std::cerr << "Inquire with fetestexcept(): ";
+#ifdef FE_INEXACT
+  if (fetestexcept(FE_INEXACT))
+    std::cerr << "FE_INEXACT ";
+#endif
+#ifdef FE_DIVBYZERO
+  if (fetestexcept(FE_DIVBYZERO))
+    std::cerr << "FE_DIVBYZERO ";
+#endif
+#ifdef FE_UNDERFLOW
+  if (fetestexcept(FE_UNDERFLOW))
+    std::cerr << "FE_UNDERFLOW ";
+#endif
+#ifdef FE_OVERFLOW
+  if (fetestexcept(FE_OVERFLOW))
+    std::cerr << "FE_OVERFLOW ";
+#endif
+#if FE_INVALID
+  if (fetestexcept(FE_INVALID))
+    std::cerr << "FE_INVALID ";
+#endif
+  std::cerr << std::endl;
+#endif // defined(PWL_HAVE_FENV_H)
+  exit(1);
+}
+
+#endif // !defined(PWL_HAVE_SIGINFO_T) || !defined(SA_SIGINFO)
+
+#endif // PWL_HAVE_DECL_SIGACTION
 
 } // namespace
 
@@ -121,19 +165,24 @@ namespace Parma_Watchdog_Library {
 
 namespace Test {
 
-#ifdef HAVE_SIGINFO_T
 void
 set_handlers() {
+#if PWL_HAVE_DECL_SIGACTION
   struct sigaction action;
-  action.sa_sigaction = fpe_handler;
   sigemptyset(&action.sa_mask);
+#if defined(PWL_HAVE_SIGINFO_T) && defined(SA_SIGINFO)
+  action.sa_sigaction = fpe_sigaction;
   action.sa_flags = SA_SIGINFO;
+#else // !defined(PWL_HAVE_SIGINFO_T) || !defined(SA_SIGINFO)
+  action.sa_handler = fpe_handler;
+  action.sa_flags = 0;
+#endif // !defined(PWL_HAVE_SIGINFO_T) || !defined(SA_SIGINFO)
   if (sigaction(SIGFPE, &action, NULL) != 0) {
     std::cerr << "sigaction() failed"
 	      << std::endl;
     abort();
   }
-#endif // defined(HAVE_SIGINFO_T)
+#endif // PWL_HAVE_DECL_SIGACTION
 
   std::set_unexpected(unexpected_exception_handler);
   std::set_terminate(uncaught_exception_handler);
