@@ -1,5 +1,5 @@
 /* Degenerate_Element enum declaration.
-   Copyright (C) 2001-2009 Roberto Bagnara <bagnara@cs.unipr.it>
+   Copyright (C) 2001-2010 Roberto Bagnara <bagnara@cs.unipr.it>
 
 This file is part of the Parma Polyhedra Library (PPL).
 
@@ -28,5 +28,10 @@ public enum Degenerate_Element {
     //! The universe element, i.e., the whole vector space.
     UNIVERSE,
     //! The empty element, i.e., the empty set.
-    EMPTY
+    EMPTY;
+
+    private static native void initIDs();
+    static {
+        initIDs();
+    }
 }
