@@ -1,5 +1,5 @@
 /* Optimization_Mode enum declaration.
-   Copyright (C) 2001-2009 Roberto Bagnara <bagnara@cs.unipr.it>
+   Copyright (C) 2001-2010 Roberto Bagnara <bagnara@cs.unipr.it>
 
 This file is part of the Parma Polyhedra Library (PPL).
 
@@ -28,5 +28,10 @@ public enum Optimization_Mode {
     //! Minimization is requested.
     MINIMIZATION,
     //! Maximization is requested.
-    MAXIMIZATION
+    MAXIMIZATION;
+
+    private static native void initIDs();
+    static {
+        initIDs();
+    }
 }
