@@ -1,5 +1,5 @@
 (* Simple program testing the PPL OCaml interface on a few random things.
-   Copyright (C) 2001-2009 Roberto Bagnara <bagnara@cs.unipr.it>
+   Copyright (C) 2001-2010 Roberto Bagnara <bagnara@cs.unipr.it>
 
 This file is part of the Parma Polyhedra Library (PPL).
 
@@ -145,13 +145,18 @@ let rec print_generator = function
       print_string_if_noisy ", ";;
 
 let print_congruence = function x,y,z ->
-   print_linear_expression x;
+  print_linear_expression x;
   print_string_if_noisy " %= ";
   print_linear_expression y;
   print_string_if_noisy " mod ";
   print_int_if_noisy(Z.to_int z);
   print_string_if_noisy ", ";;
 
+let print_artificial_parameter = function x,y ->
+  print_linear_expression x;
+  print_string_if_noisy " / ";
+  print_int_if_noisy(Z.to_int y);
+  print_string_if_noisy ", ";;
 
 (* Build linear expressions the hard way. *)
 
@@ -382,6 +387,10 @@ print_string_if_noisy "Maximum space dimension is: ";
 let i = ppl_max_space_dimension()
 in print_int_if_noisy i;;
 print_string_if_noisy "\n";;
+print_string_if_noisy "Irrational precision is: ";
+let p = ppl_irrational_precision()
+in print_int_if_noisy(p);;
+print_string_if_noisy "\n";;
 
 (* Testing exceptions *)
 try
@@ -415,6 +424,8 @@ and compute_timeout_hypercube dim_in dim_out =
   )
 in (
   begin
+    (* FIXME: in the following try/with, the timeouts should be reset
+       unconditionally. Is there a way to implement try/finally? *)
     try
       print_string_if_noisy "\nStarting ppl_reset_timeout test:\n";
       ppl_set_timeout 100;
@@ -423,9 +434,11 @@ in (
       print_string_if_noisy "ppl_reset_timeout test succeeded.\n"
     with
     | PPL_timeout_exception ->
+      ppl_reset_timeout ();
       print_string_if_noisy "ppl_reset_timeout test seems to be failed:\n";
       print_string_if_noisy "Unexpected PPL timeout exception caught.\n"
     | _ ->
+      ppl_reset_timeout ();
       print_string_if_noisy "ppl_reset_timeout test seems to be failed.";
       (* FIXME: print the contents of the exception. *)
       print_string_if_noisy "\n"
@@ -439,15 +452,218 @@ in (
       print_string_if_noisy "ppl_set_timeout test seems to be failed!\n"
     with
     | PPL_timeout_exception ->
+      ppl_reset_timeout ();
       print_string_if_noisy "ppl_set_timeout test succeded\n";
       print_string_if_noisy "Expected PPL timeout exception caught.\n"
     | _ ->
+      ppl_reset_timeout ();
       print_string_if_noisy "ppl_set_timeout test failed:\n";
       print_string_if_noisy "generic exception caught.\n"
+  end;
+  begin
+    try
+      print_string_if_noisy "\nStarting ppl_reset_deterministic_timeout test:\n";
+      ppl_set_deterministic_timeout 2000;
+      compute_timeout_hypercube 0 2;
+      ppl_reset_deterministic_timeout ();
+      print_string_if_noisy "ppl_reset_deterministic_timeout test succeeded.\n"
+    with
+    | PPL_timeout_exception ->
+      ppl_reset_deterministic_timeout ();
+      print_string_if_noisy "ppl_reset_deterministic_timeout test seems to be failed:\n";
+      print_string_if_noisy "Unexpected PPL timeout exception caught.\n"
+    | _ ->
+      ppl_reset_deterministic_timeout ();
+      print_string_if_noisy "ppl_reset_deterministic_timeout test seems to be failed.";
+      (* FIXME: print the contents of the exception. *)
+      print_string_if_noisy "\n"
+  end;
+  begin
+    try
+      print_string_if_noisy "\nStarting ppl_set_deterministic_timeout test:\n";
+      ppl_set_deterministic_timeout 2000;
+      compute_timeout_hypercube 0 100;
+      ppl_reset_deterministic_timeout ();
+      print_string_if_noisy "ppl_set_deterministic_timeout test seems to be failed!\n"
+    with
+    | PPL_timeout_exception ->
+      ppl_reset_deterministic_timeout ();
+      print_string_if_noisy "ppl_set_deterministic_timeout test succeded\n";
+      print_string_if_noisy "Expected PPL timeout exception caught.\n"
+    | _ ->
+      ppl_reset_deterministic_timeout ();
+      print_string_if_noisy "ppl_set_deterministic_timeout test failed:\n";
+      print_string_if_noisy "generic exception caught.\n"
   end
 );;
 
-(* Pointset_Powersed_Grid is not enabled by default, the following code is *)
+
+print_string_if_noisy "\nStarting PIP_Problem \n";;
+
+print_string_if_noisy "PIP_Problem 1 \n";;
+print_string_if_noisy "testing PIP_Problem_solution, ";;
+print_string_if_noisy "PIP_Problem_OK ";;
+print_string_if_noisy "and PIP_Tree_Node_is_bottom \n";;
+let pip = ppl_new_PIP_Problem 3
+             [e1 >=/ e2; e1 <=/ e2 -/ (linear_expression_of_int 7)] [2];;
+
+begin
+try
+  let ptree = ppl_PIP_Problem_solution pip
+  in let out = if (ppl_PIP_Problem_OK pip & ppl_PIP_Tree_Node_is_bottom ptree)
+                 then "success" else "failed"
+     in (print_string_if_noisy out)
+with PPL_arithmetic_overflow what ->
+  let unexpected = (ppl_Coefficient_bits() != 8)
+  in if (unexpected)
+       then raise (PPL_arithmetic_overflow(what));
+  print_string_if_noisy "Expected overflow exception caught: ";
+  print_string_if_noisy what;
+  print_string_if_noisy "\n"
+end;;
+print_string_if_noisy "\n";;
+
+print_string_if_noisy "PIP_Problem 2 \n";;
+let params = [7];;
+let pip2 = ppl_new_PIP_Problem 10 constraints1 params;;
+let i = ppl_PIP_Problem_space_dimension pip2;;
+let i = ppl_PIP_Problem_constraints pip2;;
+print_string_if_noisy "PIP_Problem constraints are \n";;
+List.iter print_constraint i;;
+print_string_if_noisy "\n";;
+ppl_PIP_Problem_add_constraint pip2 constraint1;;
+ppl_PIP_Problem_add_constraints pip2 constraints1;;
+let i = ppl_PIP_Problem_parameter_space_dimensions pip2;;
+let i = ppl_PIP_Problem_get_control_parameter pip2 Cutting_Strategy;;
+ppl_PIP_Problem_set_control_parameter pip2 Cutting_Strategy_First;;
+let i = ppl_PIP_Problem_get_control_parameter pip2 Cutting_Strategy;;
+let out = if (i == Cutting_Strategy_First)
+  then "PIP Problem Control Parameter test succeeded"
+  else "PIP Problem Control Parameter test failed"
+    in (print_string_if_noisy out);;
+print_string_if_noisy "\n";;
+let b0 = ppl_PIP_Problem_has_big_parameter_dimension pip2;;
+ppl_PIP_Problem_set_big_parameter_dimension pip2 7;;
+let big_par = ppl_PIP_Problem_get_big_parameter_dimension pip2;;
+let b1 = ppl_PIP_Problem_has_big_parameter_dimension pip2;;
+let out = if (not b0 & b1 & (big_par = 7))
+  then "ppl_PIP_Problem big parameter dimension tests succeeded"
+  else "ppl_PIP_Problem big parameter dimension tests failed"
+    in (print_string_if_noisy out);;
+print_string_if_noisy "\n";;
+
+begin
+try
+  let out = if (ppl_PIP_Problem_is_satisfiable pip2)
+    then "ppl_PIP_Problem_is_satisfiable test succeeded"
+    else "ppl_PIP_Problem_is_satisfiable test failed"
+      in (print_string_if_noisy out);
+  print_string_if_noisy "\n";
+  print_string_if_noisy "testing ppl_PIP_Problem_ascii_dump: ";
+  print_string_if_noisy (ppl_PIP_Problem_ascii_dump pip2);
+  print_string_if_noisy "\n";
+  let ptree2 = ppl_PIP_Problem_solution pip2
+  in let out2 = if (ppl_PIP_Tree_Node_is_bottom ptree2)
+    then "ppl_PIP_Tree_Node_is_bottom test succeeded"
+    else "ppl_PIP_Tree_Node_is_bottom test failed"
+      in (print_string_if_noisy out2);
+  print_string_if_noisy "\n"
+with PPL_arithmetic_overflow what ->
+  let unexpected = (ppl_Coefficient_bits() != 8)
+  in if (unexpected)
+       then raise (PPL_arithmetic_overflow(what));
+  print_string_if_noisy "Expected overflow exception caught: ";
+  print_string_if_noisy what;
+  print_string_if_noisy "\n"
+end;;
+
+print_string_if_noisy "\nPIP_Problem 3 \n";;
+let i = Variable 0
+and j = Variable 1
+and n = Variable 2
+and m = Variable 3
+and v2 = Z.from_int 2
+and v3 = Z.from_int 3
+and v4 = Z.from_int 4
+and v8 = Z.from_int 8
+and inhomogeneous n = linear_expression_of_int n
+;;
+
+let pip_c1 = v3 */ j +/ v2 */ i >=/ (inhomogeneous 8);;
+print_constraint pip_c1; print_string_if_noisy "\n" ;;
+let pip_c2 = v4 */ i -/ j >=/ (inhomogeneous 4);;
+print_constraint pip_c2; print_string_if_noisy "\n" ;;
+let pip_c3 = m >=/ j;;
+print_constraint pip_c3; print_string_if_noisy "\n" ;;
+let pip_c4 = n >=/ i;;
+print_constraint pip_c4; print_string_if_noisy "\n" ;;
+let pip_cs = [pip_c1; pip_c2; pip_c3; pip_c4] ;;
+
+let ps = [2; 3];;
+let pip3 = ppl_new_PIP_Problem 4 pip_cs ps;;
+
+begin
+try
+  let result = ppl_PIP_Problem_solve pip3 in
+  let out = if (result == Optimized_Pip_Problem)
+    then "PIP Problem_solve pip3 test succeeded"
+    else "PIP Problem_solve pip3 test failed"
+      in (print_string_if_noisy out);
+  print_string_if_noisy "\n";
+  let node = ppl_PIP_Problem_solution pip3 in
+  begin
+  let out = if (ppl_PIP_Tree_Node_OK node)
+    then "ppl_PIP_Tree_Node_OK node test succeeded"
+    else "ppl_PIP_Tree_Node_OK node test failed"
+      in (print_string_if_noisy out);
+  print_string_if_noisy "\n";
+  let out = if (ppl_PIP_Tree_Node_is_decision node)
+    then "ppl_PIP_Tree_Node_is_decision node test succeeded"
+    else "ppl_PIP_Tree_Node_is_decision node test failed"
+      in (print_string_if_noisy out);
+  print_string_if_noisy "\n";
+  let out = if (ppl_PIP_Tree_Node_artificials node = [])
+    then "ppl_PIP_Tree_Node_artificials node test succeeded"
+    else "ppl_PIP_Tree_Node_artificials node test failed"
+      in (print_string_if_noisy out);
+  print_string_if_noisy "\n";
+  let _node_cs = ppl_PIP_Tree_Node_constraints node in ();
+  let
+    _fchild = ppl_PIP_Tree_Node_false_child node
+  and
+    tchild = (ppl_PIP_Tree_Node_true_child node)
+  in (
+    let out = if (ppl_PIP_Tree_Node_is_decision tchild)
+      then "ppl_PIP_Tree_Node_is_decision tchild test succeeded"
+      else "ppl_PIP_Tree_Node_is_decision tchild test failed"
+        in (print_string_if_noisy out);
+    print_string_if_noisy "\n";
+    let
+      ttchild = (ppl_PIP_Tree_Node_true_child tchild)
+    and
+      ftchild = (ppl_PIP_Tree_Node_false_child tchild)
+    in (
+      let out = if (ppl_PIP_Tree_Node_is_solution ttchild)
+        then "ppl_PIP_Tree_Node_is_decision ttchild test succeeded"
+        else "ppl_PIP_Tree_Node_is_decision ttchild test failed"
+          in (print_string_if_noisy out);
+      print_string_if_noisy "\n";
+      let _par_vals = ppl_PIP_Tree_Node_parametric_values ttchild 0 in ();
+      let _ftchild_arts = ppl_PIP_Tree_Node_artificials ftchild in ()
+    )
+  );
+  print_string_if_noisy "\n"
+  end
+with PPL_arithmetic_overflow what ->
+  let unexpected = (ppl_Coefficient_bits() != 8)
+  in if (unexpected)
+       then raise (PPL_arithmetic_overflow(what));
+  print_string_if_noisy "Expected overflow exception caught: ";
+  print_string_if_noisy what;
+  print_string_if_noisy "\n"
+end;;
+
+(* Pointset_Powerset_Grid is not enabled by default, the following code is *)
 (* commented *)
 (* let pps = ppl_new_Pointset_Powerset_Grid_from_space_dimension 3;; *)
 (* let space_dim = ppl_Pointset_Powerset_Grid_space_dimension pps;; *)
@@ -468,5 +684,6 @@ in (
 (* print_string_if_noisy "PPS size : ";; *)
 (* let size = ppl_Pointset_Powerset_Grid_size pps;; *)
 (* print_int_if_noisy size;; *)
+
 at_exit Gc.full_major;;
 print_string_if_noisy "\nBye!\n"
