@@ -1,5 +1,5 @@
 /* Coefficient class implementation (non-inline functions).
-   Copyright (C) 2001-2009 Roberto Bagnara <bagnara@cs.unipr.it>
+   Copyright (C) 2001-2010 Roberto Bagnara <bagnara@cs.unipr.it>
 
 This file is part of the Parma Polyhedra Library (PPL).
 
@@ -42,20 +42,20 @@ const Coefficient* Coefficient_one_p = 0;
 
 void
 Coefficient_constants_initialize() {
-  assert(Coefficient_zero_p == 0);
+  PPL_ASSERT(Coefficient_zero_p == 0);
   Coefficient_zero_p = new Coefficient(0);
 
-  assert(Coefficient_one_p == 0);
+  PPL_ASSERT(Coefficient_one_p == 0);
   Coefficient_one_p = new Coefficient(1);
 }
 
 void
 Coefficient_constants_finalize() {
-  assert(Coefficient_zero_p != 0);
+  PPL_ASSERT(Coefficient_zero_p != 0);
   delete Coefficient_zero_p;
   Coefficient_zero_p = 0;
 
-  assert(Coefficient_one_p != 0);
+  PPL_ASSERT(Coefficient_one_p != 0);
   delete Coefficient_one_p;
   Coefficient_one_p = 0;
 }
