@@ -1,11 +1,11 @@
 /* Test Grid::is_pointed().
-   Copyright (C) 2001-2006 Roberto Bagnara <bagnara@cs.unipr.it>
+   Copyright (C) 2001-2010 Roberto Bagnara <bagnara@cs.unipr.it>
 
 This file is part of the Parma Polyhedra Library (PPL).
 
 The PPL is free software; you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 2 of the License, or (at your
+Free Software Foundation; either version 3 of the License, or (at your
 option) any later version.
 
 The PPL is distributed in the hope that it will be useful, but WITHOUT
@@ -68,17 +68,17 @@ test04() {
   Variable B(1);
 
   Grid gr_gs_min(2, EMPTY);
-  gr_gs_min.add_generator_and_minimize(grid_point(3*A + 2*B));
+  gr_gs_min.add_grid_generator(grid_point(3*A + 2*B));
 
   Grid gr_gs_needs_min(2, EMPTY);
-  gr_gs_needs_min.add_generator(grid_point(3*A + 2*B));
+  gr_gs_needs_min.add_grid_generator(grid_point(3*A + 2*B));
 
   Grid gr_cgs_needs_min(2);
-  gr_cgs_needs_min.add_congruence(A == 3);
-  gr_cgs_needs_min.add_congruence(B == 2);
+  gr_cgs_needs_min.add_constraint(A == 3);
+  gr_cgs_needs_min.add_constraint(B == 2);
 
-  assert(copy_compare(gr_gs_min, gr_gs_needs_min));
-  assert(copy_compare(gr_gs_needs_min, gr_cgs_needs_min));
+  // Grids gr_gs_min, gr_gs_needs_min and gr_cgs_needs_min are the
+  // same grids.
 
   bool ok = (gr_gs_min.is_discrete())
     && (gr_gs_needs_min.is_discrete())
@@ -99,26 +99,25 @@ test05() {
   Variable C(2);
 
   Grid gr_gs_min(3, EMPTY);
-  gr_gs_min.add_generator(grid_point(3*A + 2*B));
-  gr_gs_min.add_generator_and_minimize(grid_line(C));
+  gr_gs_min.add_grid_generator(grid_point(3*A + 2*B));
+  gr_gs_min.add_grid_generator(grid_line(C));
 
   Grid gr_gs_needs_min(3, EMPTY);
-  gr_gs_needs_min.add_generator(grid_point(3*A + 2*B));
-  gr_gs_needs_min.add_generator(grid_line(C));
+  gr_gs_needs_min.add_grid_generator(grid_point(3*A + 2*B));
+  gr_gs_needs_min.add_grid_generator(grid_line(C));
 
   Grid gr_cgs_needs_min(3);
-  gr_cgs_needs_min.add_congruence(A == 3);
-  gr_cgs_needs_min.add_congruence(B == 2);
+  gr_cgs_needs_min.add_constraint(A == 3);
+  gr_cgs_needs_min.add_constraint(B == 2);
 
   Grid gr_cgs_min(3);
-  gr_cgs_min.add_congruence(A == 3);
-  gr_cgs_min.add_congruence(B == 2);
+  gr_cgs_min.add_constraint(A == 3);
+  gr_cgs_min.add_constraint(B == 2);
   // Minimize the congruences.
   gr_cgs_min.minimized_congruences();
 
-  assert(copy_compare(gr_gs_min, gr_gs_needs_min));
-  assert(copy_compare(gr_gs_needs_min, gr_cgs_needs_min));
-  assert(copy_compare(gr_cgs_needs_min, gr_cgs_min));
+  // Grids gr_gs_min, gr_gs_needs_min, gr_cgs_needs_min and gr_cgs_min
+  // are the same grids.
 
   bool ok = (!gr_gs_min.is_discrete())
     && (!gr_gs_needs_min.is_discrete())
@@ -141,20 +140,20 @@ test06() {
   Variable C(2);
 
   Grid gr_gs_min(3, EMPTY);
-  gr_gs_min.add_generator(grid_point(3*A + 2*B));
-  gr_gs_min.add_generator_and_minimize(grid_point(3*A + B));
+  gr_gs_min.add_grid_generator(grid_point(3*A + 2*B));
+  gr_gs_min.add_grid_generator(grid_point(3*A + B));
 
   Grid gr_gs_needs_min(3, EMPTY);
-  gr_gs_needs_min.add_generator(grid_point(3*A + 2*B));
-  gr_gs_needs_min.add_generator(grid_point(3*A + B));
+  gr_gs_needs_min.add_grid_generator(grid_point(3*A + 2*B));
+  gr_gs_needs_min.add_grid_generator(grid_point(3*A + B));
 
   Grid gr_cgs_needs_min(3);
-  gr_cgs_needs_min.add_congruence(A == 3);
+  gr_cgs_needs_min.add_constraint(A == 3);
   gr_cgs_needs_min.add_congruence(B %= 0);
-  gr_cgs_needs_min.add_congruence(C == 0);
+  gr_cgs_needs_min.add_constraint(C == 0);
 
-  assert(copy_compare(gr_gs_min, gr_gs_needs_min));
-  assert(copy_compare(gr_gs_needs_min, gr_cgs_needs_min));
+  // Grids gr_gs_min, gr_gs_needs_min and gr_cgs_needs_min are the
+  // same grids.
 
   bool ok = (gr_gs_min.is_discrete())
     && (gr_gs_needs_min.is_discrete())
@@ -175,21 +174,21 @@ test07() {
   Variable C(2);
 
   Grid gr_gs_min(3, EMPTY);
-  gr_gs_min.add_generator(grid_point(3*A + 2*B));
-  gr_gs_min.add_generator(grid_point(3*A + B));
-  gr_gs_min.add_generator_and_minimize(grid_line(C));
+  gr_gs_min.add_grid_generator(grid_point(3*A + 2*B));
+  gr_gs_min.add_grid_generator(grid_point(3*A + B));
+  gr_gs_min.add_grid_generator(grid_line(C));
 
   Grid gr_gs_needs_min(3, EMPTY);
-  gr_gs_needs_min.add_generator(grid_point(3*A + 2*B));
-  gr_gs_needs_min.add_generator(grid_point(3*A + B));
-  gr_gs_needs_min.add_generator(grid_line(C));
+  gr_gs_needs_min.add_grid_generator(grid_point(3*A + 2*B));
+  gr_gs_needs_min.add_grid_generator(grid_point(3*A + B));
+  gr_gs_needs_min.add_grid_generator(grid_line(C));
 
   Grid gr_cgs_needs_min(3);
-  gr_cgs_needs_min.add_congruence(A == 3);
+  gr_cgs_needs_min.add_constraint(A == 3);
   gr_cgs_needs_min.add_congruence(B %= 0);
 
-  assert(copy_compare(gr_gs_min, gr_gs_needs_min));
-  assert(copy_compare(gr_gs_needs_min, gr_cgs_needs_min));
+  // Grids gr_gs_min, gr_gs_needs_min and gr_cgs_needs_min are the
+  // same grids.
 
   bool ok = (!gr_gs_min.is_discrete())
     && (!gr_gs_needs_min.is_discrete())
@@ -209,21 +208,21 @@ test08() {
   Variable B(1);
 
   Grid gr_gs_min(2, EMPTY);
-  gr_gs_min.add_generator(grid_point());
-  gr_gs_min.add_generator(grid_point(A));
-  gr_gs_min.add_generator_and_minimize(grid_point(3*A + 3*B, 4));
+  gr_gs_min.add_grid_generator(grid_point());
+  gr_gs_min.add_grid_generator(grid_point(A));
+  gr_gs_min.add_grid_generator(grid_point(3*A + 3*B, 4));
 
   Grid gr_gs_needs_min(2, EMPTY);
-  gr_gs_needs_min.add_generator(grid_point());
-  gr_gs_needs_min.add_generator(grid_point(A));
-  gr_gs_needs_min.add_generator(grid_point(3*A + 3*B, 4));
+  gr_gs_needs_min.add_grid_generator(grid_point());
+  gr_gs_needs_min.add_grid_generator(grid_point(A));
+  gr_gs_needs_min.add_grid_generator(grid_point(3*A + 3*B, 4));
 
   Grid gr_cgs_needs_min(2);
   gr_cgs_needs_min.add_congruence((4*B %= 0) / 3);
   gr_cgs_needs_min.add_congruence(A - B %= 0);
 
-  assert(copy_compare(gr_gs_min, gr_gs_needs_min));
-  assert(copy_compare(gr_gs_needs_min, gr_cgs_needs_min));
+  // Grids gr_gs_min, gr_gs_needs_min and gr_cgs_needs_min are the
+  // same grids.
 
   bool ok = (gr_gs_min.is_discrete())
     && (gr_gs_needs_min.is_discrete())
@@ -244,23 +243,23 @@ test09() {
   Variable C(2);
 
   Grid gr_gs_min(3, EMPTY);
-  gr_gs_min.add_generator(grid_point());
-  gr_gs_min.add_generator(grid_point(A));
-  gr_gs_min.add_generator(grid_line(C));
-  gr_gs_min.add_generator_and_minimize(grid_point(3*A + 3*B, 4));
+  gr_gs_min.add_grid_generator(grid_point());
+  gr_gs_min.add_grid_generator(grid_point(A));
+  gr_gs_min.add_grid_generator(grid_line(C));
+  gr_gs_min.add_grid_generator(grid_point(3*A + 3*B, 4));
 
   Grid gr_gs_needs_min(3, EMPTY);
-  gr_gs_needs_min.add_generator(grid_point());
-  gr_gs_needs_min.add_generator(grid_point(A));
-  gr_gs_needs_min.add_generator(grid_line(C));
-  gr_gs_needs_min.add_generator(grid_point(3*A + 3*B, 4));
+  gr_gs_needs_min.add_grid_generator(grid_point());
+  gr_gs_needs_min.add_grid_generator(grid_point(A));
+  gr_gs_needs_min.add_grid_generator(grid_line(C));
+  gr_gs_needs_min.add_grid_generator(grid_point(3*A + 3*B, 4));
 
   Grid gr_cgs_needs_min(3);
   gr_cgs_needs_min.add_congruence((4*B %= 0) / 3);
   gr_cgs_needs_min.add_congruence(A - B %= 0);
 
-  assert(copy_compare(gr_gs_min, gr_gs_needs_min));
-  assert(copy_compare(gr_gs_needs_min, gr_cgs_needs_min));
+  // Grids gr_gs_min, gr_gs_needs_min and gr_cgs_needs_min are the
+  // same grids.
 
   bool ok = (!gr_gs_min.is_discrete())
     && (!gr_gs_needs_min.is_discrete())
@@ -282,21 +281,21 @@ test10() {
   Variable D(3);
 
   Grid gr_gs_min(4, EMPTY);
-  gr_gs_min.add_generator(grid_point());
-  gr_gs_min.add_generator(grid_line(B));
-  gr_gs_min.add_generator_and_minimize(grid_line(C));
+  gr_gs_min.add_grid_generator(grid_point());
+  gr_gs_min.add_grid_generator(grid_line(B));
+  gr_gs_min.add_grid_generator(grid_line(C));
 
   Grid gr_gs_needs_min(4, EMPTY);
-  gr_gs_needs_min.add_generator(grid_point());
-  gr_gs_needs_min.add_generator(grid_line(B));
-  gr_gs_needs_min.add_generator(grid_line(C));
+  gr_gs_needs_min.add_grid_generator(grid_point());
+  gr_gs_needs_min.add_grid_generator(grid_line(B));
+  gr_gs_needs_min.add_grid_generator(grid_line(C));
 
   Grid gr_cgs_needs_min(4);
-  gr_cgs_needs_min.add_congruence(A == 0);
-  gr_cgs_needs_min.add_congruence(D == 0);
+  gr_cgs_needs_min.add_constraint(A == 0);
+  gr_cgs_needs_min.add_constraint(D == 0);
 
-  assert(copy_compare(gr_gs_min, gr_gs_needs_min));
-  assert(copy_compare(gr_gs_needs_min, gr_cgs_needs_min));
+  // Grids gr_gs_min, gr_gs_needs_min and gr_cgs_needs_min are the
+  // same grids.
 
   bool ok = (!gr_gs_min.is_discrete())
     && (!gr_gs_needs_min.is_discrete())
@@ -315,10 +314,10 @@ test11() {
   Variable A(0);
 
   Grid gr(3);
-  gr.add_congruence(A == 1);
-  gr.add_congruence(A == 2);
+  gr.add_constraint(A == 1);
+  gr.add_constraint(A == 2);
 
-  bool ok (gr.is_discrete());
+  bool ok = (gr.is_discrete());
 
   print_congruences(gr, "*** gr ***");
 
